/*!
 * vue-router-parameters-controller v0.10.22 
 * (c) 2023 Alex Tkachenko
 * Released under the MIT License.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueUtils = require('@aspectus/vue-utils');
var vueTag = require('@aspectus/vue-tag');

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _isPlaceholder(a) {
  return a != null && _typeof(a) === 'object' && a['@@functional/placeholder'] === true;
}

var _isPlaceholder_1 = _isPlaceholder;

/**
 * Optimized internal one-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry1(fn) {
  return function f1(a) {
    if (arguments.length === 0 || _isPlaceholder_1(a)) {
      return f1;
    } else {
      return fn.apply(this, arguments);
    }
  };
}

var _curry1_1 = _curry1;

/**
 * Optimized internal two-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry2(fn) {
  return function f2(a, b) {
    switch (arguments.length) {
      case 0:
        return f2;

      case 1:
        return _isPlaceholder_1(a) ? f2 : _curry1_1(function (_b) {
          return fn(a, _b);
        });

      default:
        return _isPlaceholder_1(a) && _isPlaceholder_1(b) ? f2 : _isPlaceholder_1(a) ? _curry1_1(function (_a) {
          return fn(_a, b);
        }) : _isPlaceholder_1(b) ? _curry1_1(function (_b) {
          return fn(a, _b);
        }) : fn(a, b);
    }
  };
}

var _curry2_1 = _curry2;

/**
 * Returns a partial copy of an object omitting the keys specified.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [String] -> {String: *} -> {String: *}
 * @param {Array} names an array of String property names to omit from the new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with properties from `names` not on it.
 * @see R.pick
 * @example
 *
 *      R.omit(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, c: 3}
 */

var omit =
/*#__PURE__*/
_curry2_1(function omit(names, obj) {
  var result = {};
  var index = {};
  var idx = 0;
  var len = names.length;

  while (idx < len) {
    index[names[idx]] = 1;
    idx += 1;
  }

  for (var prop in obj) {
    if (!index.hasOwnProperty(prop)) {
      result[prop] = obj[prop];
    }
  }

  return result;
});

var omit_1 = omit;

/**
 * Returns a partial copy of an object containing only the keys specified. If
 * the key does not exist, the property is ignored.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> {k: v}
 * @param {Array} names an array of String property names to copy onto a new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties from `names` on it.
 * @see R.omit, R.props
 * @example
 *
 *      R.pick(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, d: 4}
 *      R.pick(['a', 'e', 'f'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1}
 */

var pick =
/*#__PURE__*/
_curry2_1(function pick(names, obj) {
  var result = {};
  var idx = 0;

  while (idx < names.length) {
    if (names[idx] in obj) {
      result[names[idx]] = obj[names[idx]];
    }

    idx += 1;
  }

  return result;
});

var pick_1 = pick;

var same = function same(x) {
  return x;
};

var pathOmitter = omit_1(['path']);
var routerPicker = pick_1(['query', 'path', 'params', 'name']);

var routerOmitter = function routerOmitter(value) {
  return 'name' in value ? pathOmitter(value) : value;
};

var INTERNAL_KEY = '__internal__';
var RouterParametersController = {
  name: 'router-parameters-controller',
  props: {
    initial: {
      type: Object,
      default: function _default() {
        return {};
      }
    },
    from: {
      type: Function,
      default: same
    },
    to: {
      type: Function,
      default: same
    }
  },
  data: function data() {
    return {
      parameters: this.initial,
      hadChanged: false
    };
  },
  watch: {
    $route: {
      immediate: true,
      handler: function handler(value, oldValue) {
        this.parameters = this.from(Object.assign( // eslint-disable-line prefer-object-spread
        {}, this.initial, value.params, value.query));
        var event = {
          parameters: this.parameters
        };
        this.$emit('change', event);

        if (value.params[INTERNAL_KEY]) {
          this.$emit('change:internal', event);
          return;
        }

        if (oldValue) {
          this.$emit('change:external', event);
        }

        if (!oldValue || !this.hadChanged) {
          this.$emit('change:initial', event);
          this.hadChanged = true;
        }
      }
    }
  },
  methods: {
    updateUrl: function updateUrl(parameters) {
      var parameterKeys = Object.keys(this.$route.params);
      var paramsPicker = pick_1(parameterKeys);
      var queryGetter = this.$route.name ? omit_1(parameterKeys) : same;
      var query = this.to(parameters);
      this.$router.push(routerOmitter(routerPicker(Object.assign( // eslint-disable-line prefer-object-spread
      {}, this.$route, {
        query: queryGetter(query),
        params: Object.assign( // eslint-disable-line prefer-object-spread
        {}, this.$route.params, paramsPicker(query), _defineProperty({}, INTERNAL_KEY, true))
      })))).catch(function () {}); // For the same URL vue-router throws here.
    },
    changeParameters: function changeParameters(value) {
      this.updateUrl(value);
    }
  },
  render: function render(h) {
    return vueUtils.renderSlim(this.$scopedSlots.default({
      parameters: this.parameters,
      changeParameters: this.changeParameters,
      updateUrl: this.updateUrl
    }), h, vueTag.Tag);
  }
};

function install(Vue) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$name = _ref.name,
      name = _ref$name === void 0 ? RouterParametersController.name : _ref$name;

  Vue.component(name, RouterParametersController);
}
var main = {
  install: install
};

exports.RouterParametersController = RouterParametersController;
exports.default = main;
